@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: narray.c,v 1.5 1998/08/21 15:43:15 nickm Exp nickm $";
+static char rcsid[] = "$Id: narray.c,v 1.6 1998/08/26 01:51:28 nickm Exp $";
 /*
  *
  *  NArray - a tcl extension for manipulating multidimensional arrays
@@ -155,19 +155,20 @@ int
 NArrayCollapse(Tcl_Interp * interp, int argc, char *argv[], NArray *array1) {
   Tcl_CmdInfo cmdInfo;
   NArray *array2;
-  int i, j, listArgc, ndirs, tempval, resp;
+  int i, j, listArgc, ndirs, tempval, resp, *maskv, mask, collapsedim;
   /* Dimensions & indices - should probably size these guys dynamically */
   int direction[10], dim1[10], dim2[10]; 
   char **listArgv;
 
-  if (argc < 2) {
-    Tcl_AppendResult(interp,"Usage: arrayname collapse direction result",
-		  TCL_STATIC);
+  if ((argc != 2) && (argc != 3)) {
+    Tcl_AppendResult(interp,
+		     "Usage: arrayname collapse direction ?mask? result",
+		     TCL_STATIC);
     return TCL_ERROR;
   }
 
   /* Set up Narray2 */
-  if (Tcl_GetCommandInfo(interp,argv[1],&cmdInfo) != 1) {
+  if (Tcl_GetCommandInfo(interp,argv[argc-1],&cmdInfo) != 1) {
     Tcl_AppendResult(interp,"Cannot get info for NArray ",
 		     "\"",argv[1],"\"",(char *)NULL);
     return TCL_ERROR;
@@ -196,6 +197,7 @@ NArrayCollapse(Tcl_Interp * interp, int argc, char *argv[], NArray *array1) {
   }
 
   ndirs = 0;
+  collapsedim = 0;
   memset(direction, 0, sizeof(direction));
   memset(dim1,      0, sizeof(dim1));
   memset(dim2,      0, sizeof(dim2));
@@ -205,7 +207,10 @@ NArrayCollapse(Tcl_Interp * interp, int argc, char *argv[], NArray *array1) {
       return resp;
     }
     direction[i] = (tempval ? 1: 0);
-    if (tempval) ndirs++;
+    if (tempval) {
+      collapsedim = i;
+      ndirs++;
+    }
   }
   ckfree(listArgv);
 
@@ -239,6 +244,36 @@ NArrayCollapse(Tcl_Interp * interp, int argc, char *argv[], NArray *array1) {
     }
   }
 
+  /* Process integration mask */
+  if (argc == 3) {
+    /* Process mask list */
+    if (Tcl_SplitList(interp,argv[1],&listArgc,&listArgv) != TCL_OK) {
+      Tcl_SetResult(interp,"Enter direction as list of integers",TCL_STATIC);
+      ckfree(listArgv);
+      return TCL_ERROR;
+    }
+
+    if (listArgc != array1->dim_length[collapsedim]) {
+      Tcl_AppendResult(interp,
+		       "Number of mask elements must equal number of ",
+		       "elements along integration direction",0);
+      ckfree(listArgv);
+      return TCL_ERROR;
+    }
+    maskv = (int *) ckalloc(array1->dim_length[collapsedim]*sizeof(int));
+    for (i=0;i<array1->dim_length[collapsedim];i++) {
+      if ((resp = Tcl_GetInt(interp,listArgv[i],&maskv[i])) != TCL_OK) {
+	ckfree(listArgv);
+	ckfree(maskv);
+	return resp;
+      }      
+    }
+    ckfree(listArgv);
+  } else {
+    maskv = (int *) ckalloc(array1->dim_length[collapsedim]*sizeof(int));
+    for (i=0;i<array1->dim_length[collapsedim];i++) maskv[i] = 1;
+  }
+
   /*
    * Check storage requirements of resulting narray 
    *        - reallocate if necessary
@@ -251,7 +286,7 @@ NArrayCollapse(Tcl_Interp * interp, int argc, char *argv[], NArray *array1) {
 	    (NArrayFloat*) ckalloc(sizeof(NArrayFloat) * array2->length);
   }
   memset(array2->storage, 0, sizeof(NArrayFloat) * array2->length);
-  
+  mask = 0;
   /*
    * Now that we're set up properly, go for the whole enchilada
    *
@@ -261,25 +296,28 @@ NArrayCollapse(Tcl_Interp * interp, int argc, char *argv[], NArray *array1) {
    * 4) Calculate array2 offset from supplied indices
    * 5) Sum the two values and store in array2 element
    */
-  
+
   for (i=0;i<array1->length;i++) {
     memset(dim1,      0, sizeof(dim1));
     memset(dim2,      0, sizeof(dim2));
     IndexFromOffset(i,dim1,array1);
-    ndirs=0;
-    for (j=0;j<array1->n_dims;j++) {
-      if (!direction[j]) {
-	dim2[ndirs++] = dim1[j];
+    mask = maskv[dim1[collapsedim]];
+    if (mask) {
+      ndirs=0;
+      for (j=0;j<array1->n_dims;j++) {
+	if (!direction[j]) {
+	  dim2[ndirs++] = dim1[j];
+	}
       }
+      tempval = OffsetFromIndex(dim2,array2);
+      if (array1->debug & DEBUG_TRACE) {
+	printf("Offset = %5d\n",i); 
+	printf(" dim1:"); pdims(dim1,array1);
+	printf(" dim2:"); pdims(dim2,array2);
+	printf(" off = %5d\n",tempval);
+      }
+      *(array2->storage + tempval) += *(array1->storage + i);
     }
-    tempval = OffsetFromIndex(dim2,array2);
-    if (array1->debug & DEBUG_TRACE) {
-      printf("Offset = %5d\n",i); 
-      printf(" dim1:"); pdims(dim1,array1);
-      printf(" dim2:"); pdims(dim2,array2);
-      printf(" off = %5d\n",tempval);
-    }
-    *(array2->storage + tempval) += *(array1->storage + i);
   }
   return TCL_OK;
 }
