@@ -1,7 +1,7 @@
 #ifndef narray_h
 #define narray_h
 
-/* $Id: narray.h,v 1.1 1998/08/20 16:54:26 nickm Exp nickm $ */
+/* $Id: narray.h,v 1.3 1998/08/26 18:48:06 nickm Exp nickm $ */
 
 /*
  * This software is copyright (C) 1994 by the Lawrence Berkeley Laboratory.
@@ -27,12 +27,9 @@
 
 #include "tcl.h"
 
-#define NARRAY_VERSION "0.8"
+#define NARRAY_VERSION "0.81"
 
 typedef float NArrayFloat;
-#ifdef WITH_NETCDF
-#define NARRAY_NCFLOAT NC_FLOAT
-#endif
 
 typedef struct _NArray NArray;
 
@@ -96,11 +93,6 @@ struct _NArray {
     int length;			/* length of storage */
     NArrayFloat* storage;
     int debug;			/* DEBUG_* flags */
-#ifdef WITH_NETCDF
-    int ncopts;			/* pushed to global ncopts before netCDF
-				 * calls
-				 */
-#endif
     char* errmsg;		/* error messages from code eval's */
     NArray_Closure closure;
     int dim_length[1];		/* this struct is extended so dim_length
